//+------------------------------------------------------------------+
//|                                                    Supremacy.mq4 |
//|                                         Copyright 2015, nicholas |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, nicholas"
#property link      ""
#property version   "1.07"
#property strict

#include <stderror.mqh> 
#include <stdlib.mqh>

input string   _generalParameters         = "===   ===";
input string   Pairs                      = "EURUSD, USDCHF, USDCAD, GBPUSD, AUDUSD, NZDUSD, EURJPY, AUDJPY, USDJPY, GBPJPY";
input string   PairsSufix                 = "";
input int      TimeFrame                  = 1440;
input int      CheckFromBarStartSeconds   = 300;
input int      CheckToBarEndSeconds       = 300;
input int      MinOpenPercent             = 0;
input int      MaxOpenPercent             = 40;


input string   _url                       = "=== MyFxBook parameters ===";
input string   MyFxBookUrl                = "http://www.myfxbook.com/community/outlook"; // MyFxbook URL    
input int      Timeout                    = 1000; //  .  1000 (1 .)     
input int      DelayAfterFail             = 30;
input int      CheckStatsIntervalSeconds  = 3600;
input int      ExpireDataInSeconds        = 300;
input bool     DisplayStats               = true;
input bool     ShowSwaps                  = false;

input string   _order                     = "===  ===";
input int      MMMethod                   = 1;        // 0 -    (Lots), 1 -   
input double   Lots                       = 0.01;     //    MinDepo  MMMethod=0     MMMethod=1
input double   MinDepo                    = 1000;     //       MMMethod=1.   ( ).        DepoStep     LotsStep
input double   DepoStep                   = 1000;     //       LotsStep
input double   LotsStep                   = 0.01;     //  
extern int     MarketStopLoss             = 100;
extern int     MinLimitOrderDistance      = 10;
extern int     LimitStopLoss              = 50;
extern int     TakeProfit                 = 0;
input bool     OpenLimitOrders            = true;
input bool     ConsiderSwaps              = true;

input string   _other                     = "===   ===";
input int      Magic                      = 274204;   // .             .
extern int     Slippage                   = 3;        //  
input bool     Auto5Digits                = true;     //   5-  .         10    5-  
input bool     DownloadErrorAlert         = true;
input string   EaComment                  = "Supremacy v1.07"; //  

//--------------- TEST PARAMETERS --------------------
int TestShortPercent = 70;
int TestLongPercent = 30;
double TestShortSwap = 1;
double TestLongSwap = 1;
//----------------------------------------------------

int FromBarStartWorkInterval    = 240;

bool isFromBarStartTimeGood = false;
bool isToBarEndTimeGood = false;

int retryAttempts = 10;

int workTimeFrame;

string myfxbookPairs[], terminalPairs[];
int longPercentages[];
int shortPercentages[];
int workSize;

datetime lastPercentagesUpdate = 0;

int allOrdersCount, 
   marketOrdersCount, stopOrdersCount, limitOrdersCount,
   buyOrdersCount, sellOrdersCount, sellStopOrdersCount, buyStopOrdersCount,
   buyLimitOrdersCount, sellLimitOrdersCount;
   
int downloadRetries = 10;
int OnInit()
{
   if(Auto5Digits)
   {
      if(Digits == 5 || Digits == 3)
      {         
         TakeProfit *= 10;
         MarketStopLoss *= 10;    
         LimitStopLoss *= 10;
         Slippage *= 10;
         MinLimitOrderDistance *= 10;     
      }
   }    
   
   workTimeFrame = (TimeFrame <= 0)? Period(): TimeFrame;
   string tempPairs[];
   if(IsTesting() || StringLen(Pairs) == 0)
   {
      ArrayResize(tempPairs, 1);
      tempPairs[0] = Symbol();
   }
   else
   {
      StringSplit(Pairs, ',', tempPairs);
   }
   
   workSize = 0;
   ArrayResize(myfxbookPairs, ArraySize(tempPairs));
   ArrayResize(terminalPairs, ArraySize(tempPairs));
   
   for(int i=0; i < ArraySize(tempPairs); i++)
   {
      StringToUpper(tempPairs[i]);
      tempPairs[i] = StringTrimLeft(StringTrimRight(tempPairs[i]));
      
      string checkReal = StringConcatenate(tempPairs[i], PairsSufix);
      
      if(!IsRealSymbol(checkReal))
      {
         Print("Pair ", checkReal, " doesn't exists in the terminal");
         continue;
      }      
      
      myfxbookPairs[workSize] = tempPairs[i];
      terminalPairs[workSize] = checkReal;
      
      Print("myfxbookPairs[", workSize, "]=", myfxbookPairs[workSize], " in terminal: ", checkReal);
      workSize++;
   }
     
   ArrayResize(longPercentages, workSize);
   ArrayResize(shortPercentages, workSize);
   ArrayResize(myfxbookPairs, workSize);
   ArrayResize(terminalPairs, workSize);      
   
   return(INIT_SUCCEEDED);
}
void OnDeinit(const int reason)
{
   
}

void OnTick()
{      
   UpdatePercentages(); 
   
   CheckBarTimes();   
   
   if(!isToBarEndTimeGood && !isFromBarStartTimeGood)
      return;
   
   if(!UpdatePercentages(true))
      return;   
   
   if(isToBarEndTimeGood)
   {
      DeletePendingOrders();
      if((DayOfWeek() >= 5 || DayOfYear() >= 350))
      {
         CloseOrders();       
         return;
      }
      
   }
      
   RefreshRates();
   
   for(int i=0; i < workSize; i++)
   {      
      ProcessPair(i);
   }                          
}

//+------------------------------------------------------------------+
void CheckFromBarStartTime()
{
   static datetime lastFromBarStartTimeCheck = 0;
   
   isFromBarStartTimeGood = false;
   
   datetime currBarTime = iTime(NULL, workTimeFrame, 0);
   
   if(lastFromBarStartTimeCheck >= currBarTime)
      return;
   
   
   int secondsFromBarStart = CurTime() - currBarTime;
   
   if(secondsFromBarStart < CheckFromBarStartSeconds)
      return;   
      
   if(secondsFromBarStart > (CheckFromBarStartSeconds + FromBarStartWorkInterval))
      return;
      
   isFromBarStartTimeGood = true;
   lastFromBarStartTimeCheck = currBarTime;   
}
//+------------------------------------------------------------------+
void CheckToBarEndTime()
{
   static datetime lastToBarEndTimeCheck = 0;
   
   isToBarEndTimeGood = false;
   
   datetime currBarTime = iTime(NULL, workTimeFrame, 0);
   
   if(lastToBarEndTimeCheck >= currBarTime)
      return;
                           
   int secondsToBarEnd = currBarTime + workTimeFrame*60 - CurTime();
   
   if(secondsToBarEnd > CheckToBarEndSeconds)
      return;
                     
   isToBarEndTimeGood = true;
   lastToBarEndTimeCheck = currBarTime;   
}

//+------------------------------------------------------------------+
void CheckBarTimes()
{      
   CheckFromBarStartTime();
   CheckToBarEndTime();
}

//+------------------------------------------------------------------+
void ShowStats()
{
   string comment = "==========================";
   for(int i=0; i < workSize; i++)
   {
      comment += "\n" + myfxbookPairs[i]; 
      comment += " L: " + longPercentages[i] + "%"; 
      comment += " S: " +shortPercentages[i] + "%";
      double pairAsk = MarketInfo(terminalPairs[i], MODE_ASK);
      double pairBid = MarketInfo(terminalPairs[i], MODE_BID);
      double pairPoint = MarketInfo(terminalPairs[i], MODE_POINT);
      comment += " Spread:" + DoubleToStr((pairAsk-pairBid)/pairPoint, 2);
            
      if(ShowSwaps)
      {
         comment += " SwapL:" + DoubleToStr(GetLongSwap(i), 2);
         comment += " SwapSh:" + DoubleToStr(GetShortSwap(i), 2);      
      }
   }
   comment += "\nLast Update: " + TimeToString(lastPercentagesUpdate);
   
   Comment(comment);
}
//+------------------------------------------------------------------+
bool UpdatePercentages(bool isWorkUpdate=false)
{        
   
   if(IsTesting())
   {
      shortPercentages[0] = TestShortPercent;
      longPercentages[0] = TestLongPercent;
      return true;
   }
   
   int secondsFromLastUpdate = TimeCurrent() - lastPercentagesUpdate;
   if(isWorkUpdate)
   {
      if(secondsFromLastUpdate < ExpireDataInSeconds)
         return true;
      
   }
   else if(secondsFromLastUpdate < CheckStatsIntervalSeconds)
      return true;
   
   bool isOk=true;
   for(int i=0; i < downloadRetries; i++)
   {
      if(!isOk)
      {
         Sleep(DelayAfterFail*1000);
      }
      
      isOk = true;
      
      uint start=GetTickCount();
      char html[];
      if(!DownloadData(html))
      {         
         Print("Can't download data from myfxbook using:", MyFxBookUrl);
         isOk = false;         
         continue;
      }
         
      uint downloadTime=GetTickCount()-start;
      PrintFormat("Downloading mysqldata took %d ms", downloadTime);            
   
      isOk = true;
      for(int j=0; j < workSize && isOk; j++)
      {
         if(!GetPercentages(html, myfxbookPairs[j], shortPercentages[j], longPercentages[j]))
         {
            Print("Can't get percentages for pair '", myfxbookPairs[j], "'");
            isOk = false;            
         }
         
         if(shortPercentages[j] < 0 || longPercentages[j] < 0)
         {
            Print("Incorrect percentages for pair '", myfxbookPairs[j], "'");
            isOk = false;            
         }         
      }
      
      if(isOk)
         break;         
   }        
       
   if(!isOk)
   {
      if(DownloadErrorAlert)      
      {
         Alert("     : ", MyFxBookUrl);
      }  
            
      return false;
   }
        
   lastPercentagesUpdate = TimeCurrent();
           
   if(DisplayStats)
      ShowStats();
      
   return true;                  
}
//+------------------------------------------------------------------+
bool IsPercentGood(double percent)
{
   if(percent <= 0)
      return false;
   
   if(percent < MinOpenPercent)
      return false;
         
   if(percent > MaxOpenPercent)
      return false;
           
   return true;
}

//+------------------------------------------------------------------+
void TryOpenLongOrder(int pairIndex)
{
   if(!IsPercentGood(longPercentages[pairIndex]))
      return;         
      
   double TP = 0, SL = 0;
   double pairAsk = MarketInfo(terminalPairs[pairIndex], MODE_ASK);
   double pairPoint = MarketInfo(terminalPairs[pairIndex], MODE_POINT);

   if(MarketStopLoss > 0)
      SL = pairAsk - MarketStopLoss*pairPoint;
   if(TakeProfit > 0)
      TP = pairAsk + TakeProfit*pairPoint;

   double lots = GetLots(pairIndex);      
   if(lots >= 0)      
      OrderSendReliable(terminalPairs[pairIndex], OP_BUY, lots, pairAsk, Slippage, SL, TP, EaComment, Magic, 0, Green); 
}
//+------------------------------------------------------------------+
void TryOpenShortOrder(int pairIndex)
{ 
   if(!IsPercentGood(shortPercentages[pairIndex]))
      return;   
      
   double TP = 0, SL = 0;
   double pairBid = MarketInfo(terminalPairs[pairIndex],MODE_BID);
   double pairPoint = MarketInfo(terminalPairs[pairIndex], MODE_POINT);
   
   if(MarketStopLoss > 0)
      SL = pairBid + MarketStopLoss*pairPoint;
   if(TakeProfit > 0)
      TP = pairBid - TakeProfit*pairPoint;
   
   double lots = GetLots(pairIndex);      
   if(lots >= 0)      
      OrderSendReliable(terminalPairs[pairIndex], OP_SELL, lots, pairBid, Slippage, SL, TP, EaComment, Magic, 0, Red);  
}
//+------------------------------------------------------------------+
void TrySetBuyLimitOrder(int pairIndex)
{ 
   if(!IsPercentGood(longPercentages[pairIndex]))
      return;

   double TP = 0, SL = 0;
   double prevLow = iLow(terminalPairs[pairIndex], workTimeFrame, 1);
   double pairAsk = MarketInfo(terminalPairs[pairIndex], MODE_ASK);
   double pairPoint = MarketInfo(terminalPairs[pairIndex], MODE_POINT);
   
   if((pairAsk - prevLow) <= MinLimitOrderDistance*pairPoint)
   {
      Print("Can't open Buy Limit Order for ", terminalPairs[pairIndex], ": previous Low less than ", MinLimitOrderDistance," pips far from Ask price");
      return;
   }   
   
   if(LimitStopLoss > 0)
      SL = prevLow - LimitStopLoss*pairPoint;
   if(TakeProfit > 0)
      TP = prevLow + TakeProfit*pairPoint;

   double lots = GetLots(pairIndex);      
   if(lots >= 0)      
      OrderSendReliable(terminalPairs[pairIndex], OP_BUYLIMIT, lots, prevLow, Slippage, SL, TP, EaComment + " Limit", Magic, 0, Green);
}
//+------------------------------------------------------------------+
void TrySetSellLimitOrder(int pairIndex)
{   
   if(!IsPercentGood(shortPercentages[pairIndex]))
      return;
   
   double TP = 0, SL = 0;
   
   double pairBid = MarketInfo(terminalPairs[pairIndex],MODE_BID);
   
   double prevHigh = iHigh(terminalPairs[pairIndex], workTimeFrame, 1);
   double pairPoint = MarketInfo(terminalPairs[pairIndex], MODE_POINT);
   
   if((prevHigh - pairBid) <= MinLimitOrderDistance*pairPoint )
   {
      Print("Can't open Sell Limit Order for ", terminalPairs[pairIndex], ": previous High is less than ", MinLimitOrderDistance ," pips far from current Bid price");
      return;
   }   
   
   if(LimitStopLoss > 0)
      SL = prevHigh + LimitStopLoss*pairPoint;
   if(TakeProfit > 0)
      TP = prevHigh - TakeProfit*pairPoint;
   
   double lots = GetLots(pairIndex);      
   if(lots >= 0)   
      OrderSendReliable(terminalPairs[pairIndex], OP_SELLLIMIT, lots, prevHigh, Slippage, SL, TP, EaComment + " Limit", Magic, 0, Red); 
}
//+------------------------------------------------------------------+
double GetLongSwap(int pairIndex)
{
   if(IsTesting())
      return TestLongSwap;
    
    return MarketInfo(terminalPairs[pairIndex], MODE_SWAPLONG);   
}
//+------------------------------------------------------------------+
double GetShortSwap(int pairIndex)
{
   if(IsTesting())
      return TestShortSwap;
    
    return MarketInfo(terminalPairs[pairIndex], MODE_SWAPSHORT);
}
//+------------------------------------------------------------------+
void ProcessPair(int pairIndex)
{
   double swapLong = -1, swapShort = -1;
   
   if(ConsiderSwaps)
   {
      swapLong = GetLongSwap(pairIndex);
      swapShort = GetShortSwap(pairIndex);
   }
   
   OrdersCount(terminalPairs[pairIndex]);
   
   if(isFromBarStartTimeGood)
   {
      if(OpenLimitOrders)
      {
         TrySetBuyLimitOrder(pairIndex);
         TrySetSellLimitOrder(pairIndex);                 
      }
      
      datetime deleteBeforeTime = TimeCurrent() - CheckFromBarStartSeconds - CheckToBarEndSeconds - FromBarStartWorkInterval;
      if(swapLong > 0)
         CloseOrders(terminalPairs[pairIndex], OP_BUY, deleteBeforeTime);
            
      if(swapLong <= 0 || buyOrdersCount== 0)
         TryOpenLongOrder(pairIndex);
         
      if(swapShort > 0)
         CloseOrders(terminalPairs[pairIndex], OP_SELL, deleteBeforeTime);
            
      if(swapShort <= 0 || sellOrdersCount == 0)
         TryOpenShortOrder(pairIndex);      
   }
   
   if(isToBarEndTimeGood)
   {      
      if(swapLong > 0)
         TryOpenLongOrder(pairIndex);
      else         
         CloseOrders(terminalPairs[pairIndex], OP_BUY);
         
      if(swapShort > 0)
         TryOpenShortOrder(pairIndex);
      else         
         CloseOrders(terminalPairs[pairIndex], OP_SELL);               
   }   
}
//+------------------------------------------------------------------+
bool IsNewBar(int timeFrame) 
{
	static datetime lastBarTime;
	if (lastBarTime == iTime(NULL, timeFrame, 0)) 
		return (false);
	lastBarTime = iTime(NULL, timeFrame, 0);
	return (true);
}
//+------------------------------------------------------------------+
double GetLots(int pairIndex) 
{
    if(MMMethod == 0)
      return (Lots);
      
   if(AccountFreeMargin() < MinDepo)
      return(Lots);
      
   double lot=Lots+MathFloor((AccountFreeMargin()-MinDepo)/DepoStep)*LotsStep;   
   
   return (NormalizeLots(pairIndex, lot, false));   
}

//+------------------------------------------------------------------+
bool DownloadData(char &result[])
{
   string resultHeaders;
   char post[];
   
   ResetLastError();   
   
   if(WebRequest("GET", MyFxBookUrl, NULL, NULL, Timeout, post, 0, result, resultHeaders) < 0)
   {
      Print("  WebRequest.    =",GetLastError());    
      Print("    '", MyFxBookUrl, "'    URL    ->->, ''");
      return false;
   }
        
   return true;
}

//+------------------------------------------------------------------+
int StringFindInArray(const char& arrayValue[], string matchSubstring, int startPos = 0)
{
   char matchSubstringArray[];
   //StringToCharArray(matchSubstring, matchSubstringArray);

   for(int i=startPos; i < ArraySize(arrayValue); i++)
   {
      if(arrayValue[i] == StringGetCharacter(matchSubstring, 0))
      {
         bool found = true;
         for(int j = 1; j < StringLen(matchSubstring) && found; j++)
         {
            if(arrayValue[i+j] != StringGetCharacter(matchSubstring, j))
               found = false;
         }  
         
         if(found)
         {
           // Print("Found i=", i);
            return i;
         }
      }
   }
   
   return -1;
}

int GetNearPercent(const char& charArray[], string side, int startIndex)
{
   int sideIndex = StringFindInArray(charArray, side, startIndex);
   if(sideIndex < 0)
      return -1;
      
   if((sideIndex - startIndex) > 1500)
      return -1;
      
   int percentTagIndex = StringFindInArray(charArray, "<td>", sideIndex);   
   if(percentTagIndex < 0)
      return -1;
   
   string percent = "";
   for(int i=4; i <= 6; i++)
   {
      char c = charArray[percentTagIndex+i];
      if( c < '0' || c > '9')
         break;
      percent += CharToString(c);
   }   
   
   if(percent == "")
      return -1;
      
   return StringToInteger(percent);
}
//+------------------------------------------------------------------+
bool GetPercentages(const char& charArray[], string pair, int &shortPercent, int &longPercent)
{
   shortPercent = -1;
   longPercent = -1;
   int pairBlockIndex = StringFindInArray(charArray, StringConcatenate("symbolNameCell", pair) );
   if(pairBlockIndex < 0)
   {
      Print("Error: can't find info for pair '", pair, "'");
      return false;
   }
      
   shortPercent = GetNearPercent(charArray, "Short", pairBlockIndex);
   longPercent = GetNearPercent(charArray, "Long", pairBlockIndex);
   
   return true;    
}
//+------------------------------------------------------------------+
void OrdersCount(string pair) 
{
   buyOrdersCount = 0;
   sellOrdersCount = 0;
   sellStopOrdersCount = 0;
   buyStopOrdersCount = 0;
   sellLimitOrdersCount = 0;
   buyLimitOrdersCount = 0;
   allOrdersCount = 0;      
   
   for (int i = OrdersTotal() - 1; i >= 0; i--) {
      if (OrderSelect(i, SELECT_BY_POS)) {
         if(pair != "")
            if(OrderSymbol() != pair)
               continue;
            
         if(OrderMagicNumber() != Magic) 
            continue;
            
         if (OrderType() == OP_BUY)
         {
            buyOrdersCount++;            
         }
         if (OrderType() == OP_SELL)
         {
            sellOrdersCount++;            
         }
         if (OrderType() == OP_SELLSTOP)
         {
            sellStopOrdersCount++;            
         }
         if (OrderType() == OP_BUYSTOP)
         {
            buyStopOrdersCount++;            
         }
         if (OrderType() == OP_BUYLIMIT)
         {
            buyLimitOrdersCount++;            
         }
         if (OrderType() == OP_SELLLIMIT)
         {
            sellLimitOrdersCount++;            
         }         
      }
   }
   
   allOrdersCount = buyOrdersCount + sellOrdersCount + sellStopOrdersCount + buyStopOrdersCount + buyLimitOrdersCount + sellLimitOrdersCount;
   marketOrdersCount = buyOrdersCount + sellOrdersCount;
   stopOrdersCount = sellStopOrdersCount + buyStopOrdersCount;
   limitOrdersCount = buyLimitOrdersCount + sellLimitOrdersCount;
}

//+------------------------------------------------------------------+
void DeletePendingOrders(string pair = "")
{      
   DeleteOrdersByType(OP_BUYLIMIT, pair);
   DeleteOrdersByType(OP_SELLLIMIT, pair);   
}

//+------------------------------------------------------------------+
void DeleteOrdersByType(int orderType, string pair, datetime beforeTime = EMPTY_VALUE) 
{       
   if (OrdersTotal() <= 0)
      return;
        
   for (int i = OrdersTotal() - 1; i >=0 ; i--) 
   {
      if(!OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
         continue;
      
      if(pair != "")
         if (OrderSymbol() != pair)
            continue;  
         
      if(OrderMagicNumber() != Magic)
         continue;
            
      if(OrderType() != orderType)
         continue;      
      
      if(OrderOpenTime() > beforeTime)
         continue;
         
      OrderDeleteReliable(OrderTicket());      
   }  
}

//+------------------------------------------------------------------+
void CloseOrders(string pair = "", int orderType = -1, datetime beforeTime = EMPTY_VALUE)
{
   if (OrdersTotal() <= 0)
      return;
        
   for (int i = OrdersTotal() - 1; i >=0 ; i--) {
      if(!OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
         continue;      
         
      if(OrderMagicNumber() != Magic)
         continue;
               
      if(pair != "")
         if (OrderSymbol() != pair)
            continue; 
                     
      if(beforeTime != EMPTY_VALUE)
      {
         Print("beforeTime=", beforeTime);
         Print("OrderOpenTime=", OrderOpenTime());
         if(OrderOpenTime() >= beforeTime)
            continue;
      }

      if(orderType >= 0)
         if(OrderType() != orderType)
            continue;

      
      if(OrderType() == OP_BUY)
         OrderCloseReliable(OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_BID), Slippage, Blue);      
         
      if(OrderType() == OP_SELL)      
         OrderCloseReliable(OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_ASK), Slippage, Red);
   }  
}

//+------------------------------------------------------------------+
int OrderSendReliable(string symbol, int cmd, double lots, double price, int slippage, double stoploss, double takeprofit, string comment = "", int magic = 0, int expiration = 0, color arrow_color = CLR_NONE) 
{   
   Print("OrderSendReliable(", symbol, ",", cmd, ",", lots, ",", price, ",", slippage, ",", stoploss, ",", takeprofit, ",", comment, ",", magic, ",", expiration, ",", arrow_color, ")");   
   
   while (true) {
      if (IsStopped()) 
      {
         Print("OrderSendReliable(): Trading is stopped!");
         return (-1);
      }
      RefreshRates();
      if (cmd == OP_BUY) price = MarketInfo(symbol, MODE_ASK);
      if (cmd == OP_SELL) price = MarketInfo(symbol, MODE_BID);
      if (!IsTradeContextBusy()) {
         int symbolDigits = (int) MarketInfo(symbol, MODE_DIGITS);
         int ticket = OrderSend(symbol, cmd, lots, NormalizeDouble(price, symbolDigits), slippage, NormalizeDouble(stoploss, symbolDigits), NormalizeDouble(takeprofit, symbolDigits), comment, magic, expiration, arrow_color);
         if (ticket > 0) {
            Print("OrderSendReliable(): Success! Ticket: ", ticket);
            return (ticket);
         }
         int error = GetLastError();
         if (IsTemporaryError(error)) 
            Print("OrderSendReliable(): Temporary Error: ", error, " ", ErrorDescription(error), ". waiting.");
         else {
            Print("OrderSendReliable(): Permanent Error: ", error, " ", ErrorDescription(error), ". giving up.");
            return (-1);
         }
      } else 
         Print("OrderSendReliable(): Must wait for trade context");
         
      Sleep(MathRand() / 10);
   }
   
   return (-1);
}

//+------------------------------------------------------------------+
bool OrderDeleteReliable(int ticket, color arrow_color = CLR_NONE) {   
   Print("OrderDeleteReliable(" + ticket + ")");
   
   for (int it=1; it<=retryAttempts; it++) 
   {
      if (!IsTesting() && (!IsExpertEnabled() || IsStopped()))
      {
         Print("OrderDeleteReliable(" + ticket + "): Trading is stopped!");
         return (false);
      }
      
      RefreshRates();      
      if (!IsTradeContextBusy()) 
      {
         if (OrderDelete(ticket, arrow_color)) 
         {
            Print("OrderDeleteReliable(" + ticket + "): Success!");
            return (true);
         }
         int error = GetLastError();
         if (IsTemporaryError(error)) 
            Print("OrderDeleteReliable(" + ticket + "): Temporary Error: " + error + " " + ErrorDescription(error) + ". waiting.");
         else 
         {
            Print("OrderDeleteReliable(" + ticket + "): Permanent Error: " + error + " " + ErrorDescription(error) + ". giving up.");
            return (false);
         }
      } 
      else 
         Print("OrderDeleteReliable(" + ticket + "): Must wait for trade context");
      Sleep(MathRand() / 10);
   }
   return (true);
}
//+------------------------------------------------------------------+

bool OrderCloseReliable(int ticket, double lots, double price, int slippage, color arrow_color = CLR_NONE) {   
   Print("OrderCloseReliable(", ticket, ", ", lots, ", ", price, ", ", slippage, ", ", arrow_color, ")");
   OrderSelect(ticket, SELECT_BY_TICKET, MODE_TRADES);
   while (true) {
      if (IsStopped()) {
         Print("OrderCloseReliable(" + ticket + "): Trading is stopped!");
         return (false);
      }
      RefreshRates();
      if (OrderType() == OP_BUY) price = MarketInfo(OrderSymbol(), MODE_BID);
      if (OrderType() == OP_SELL) price = MarketInfo(OrderSymbol(), MODE_ASK);
      Print("OrderCloseReliable: orderType=", OrderType(), " price=", price);
      
      if (!IsTradeContextBusy()) {         
         if (OrderClose(ticket, lots, NormalizeDouble(price, MarketInfo(OrderSymbol(), MODE_DIGITS)), slippage, arrow_color)) {
            Print("OrderCloseReliable(" + ticket + "): Success!");
            return (true);
         }
         int error = GetLastError();
         if (IsTemporaryError(error)) 
            Print("OrderCloseReliable(" + ticket + "): Temporary Error: " + error + " " + ErrorDescription(error) + ". waiting.");
         else {
            Print("OrderCloseReliable(" + ticket + "): Permanent Error: " + error + " " + ErrorDescription(error) + ". giving up.");
            return (false);
         }
      } else 
         Print("OrderCloseReliable(" + ticket + "): Must wait for trade context");
      Sleep(MathRand() / 10);
   }
   return (true);
}
//+------------------------------------------------------------------+
bool IsTemporaryError(int error) {
   return (error == ERR_NO_ERROR
		|| error == ERR_COMMON_ERROR
		|| error == ERR_SERVER_BUSY
		|| error == ERR_NO_CONNECTION
		|| error == ERR_MARKET_CLOSED
		|| error == ERR_PRICE_CHANGED
		|| error == ERR_INVALID_PRICE
		|| error == ERR_OFF_QUOTES
		|| error == ERR_BROKER_BUSY
		|| error == ERR_REQUOTE
		|| error == ERR_TRADE_TIMEOUT
		|| error == ERR_TRADE_CONTEXT_BUSY
	);
}

//+------------------------------------------------------------------+
double NormalizeLots(int pairIndex, double lots, bool useMaxLot = true){
   lots -= MarketInfo(terminalPairs[pairIndex], MODE_MINLOT);   
   lots /= MarketInfo(terminalPairs[pairIndex], MODE_LOTSTEP);   
   lots = MathRound(lots);   
   lots *= MarketInfo(terminalPairs[pairIndex], MODE_LOTSTEP);   
   lots += MarketInfo(terminalPairs[pairIndex], MODE_MINLOT);   
   lots = NormalizeDouble(lots, 6);   
   
   /*if(useMaxLot && MartinMaxLots > 0)
      lots = MathMin(lots, MartinMaxLots);      */
      
   lots = MathMin(lots, MarketInfo(terminalPairs[pairIndex], MODE_MAXLOT));   
   lots = MathMax(lots, MarketInfo(terminalPairs[pairIndex], MODE_MINLOT));   
   
   return(lots);
}

//+------------------------------------------------------------------+
double ND(string symbol, double value)
{
   int symbolDigits = (int) MarketInfo(symbol, MODE_DIGITS);
   return (NormalizeDouble(value, symbolDigits));
}

//+------------------------------------------------------------------+
bool IsRealSymbol( string symbol )
{
  return(MarketInfo(symbol, MODE_BID) != 0);
}
